@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.70p1)
- *  Copyright 2000 Cornell University
+ *  Ensemble, 1.10
+ *  Copyright 2001 Cornell University, Hebrew University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -32,6 +32,13 @@ val default_info :
   debug ->				(* application name *)
   View.full
 
+(* Take View.full and switch on all the security switches. 
+ *)
+val set_secure : 
+  View.full -> 
+  Property.id list -> 
+  View.full
+
 (* Create view state information for singleton groups, but
  * you can give more information than default_info.
  *)
