@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.70p1)
- *  Copyright 2000 Cornell University
+ *  Ensemble, 1.10
+ *  Copyright 2001 Cornell University, Hebrew University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -24,10 +24,12 @@ type c_dncall =
   | C_Cast of Buf.t
   | C_Send of endpt * Buf.t
   | C_Suspect of endpt array 
+  | C_XferDone of unit
   | C_Protocol of string
   | C_Properties of string
   | C_Leave of unit
   | C_Prompt of unit
+  | C_Rekey of unit
   | C_BlockOk of unit
   | C_Void of unit
 
