@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.70p1)
- *  Copyright 2000 Cornell University
+ *  Ensemble, 1.10
+ *  Copyright 2001 Cornell University, Hebrew University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -18,12 +18,12 @@ open Layer
 open View
 (**************************************************************)
 let name = Trace.file "CONFIG_TRANS"
-let failwith = Trace.make_failwith name
+let failwith s = Trace.make_failwith name s
+let log = Trace.log name 
 (**************************************************************)
 let name_send = "CONFIG_TRANS(send)"
 let name_cast = "CONFIG_TRANS(cast)"
 let name_recv = "CONFIG_TRANS(recv)"
-let name_mrge = "CONFIG_TRANS(mrge)"
 let name_goss = "CONFIG_TRANS(goss)"
 (**************************************************************)
 
@@ -45,23 +45,28 @@ let gossip_marsh mbuf =
       None ev
     in
 
+    let merge = Event.getExtender
+      (function Event.MergeGos(a,b,c,d) -> (Some (Some (a,b,c,d))) | _ -> None)
+      None ev
+    in
+
     let dbg_name = Event.getExtender
       (function Event.DbgName name -> (Some (Some name)) | _ -> None)
       None ev
     in
 
-    match exchange,switch,heal,dbg_name with
-    | None,None,None,None -> None
+    match exchange,switch,heal,merge,dbg_name with
+    | None,None,None,None,None -> None
     | _ ->
 	let msg = Marsh.marsh_init mbuf in
 	Marsh.write_option msg (Marsh.write_string msg) exchange ;
-	let iovl = marsh (switch,heal,dbg_name) in
+	let iovl = marsh (switch,heal,merge,dbg_name) in
 	Marsh.write_iovl_len msg iovl ;
 	let iovl = Marsh.marsh_done msg in
 	Some iovl
 
   and unpack_gossip secure iovl =
-    let exchange,switch,heal,dbg_name =
+    let exchange,switch,heal,merge,dbg_name =
       try
 	let msg = Marsh.unmarsh_init mbuf iovl in
 
@@ -71,25 +76,26 @@ let gossip_marsh mbuf =
 
 	(* Only read the other fields if things are secure.
 	 *)
-	let switch,heal,dbg_name = 
+	let switch,heal,merge,dbg_name = 
 	  if secure then (
 	    let iovl = Marsh.read_iovl_len msg in
 	    unmarsh iovl
-	  ) else (None,None,None)
+	  ) else (None,None,None,None)
 	in
 
 	Marsh.unmarsh_done msg ;
-	(exchange,switch,heal,dbg_name)
+	(exchange,switch,heal,merge,dbg_name)
       with Marsh.Error s -> 
 	eprintf "CONFIG_TRANS:unpack_gossip:bad gossip format:%s (continuing)\n" s ;
-	(None,None,None,None)
+	(None,None,None,None,None)
     in
 
-    let exchange = option_map (fun (a)       -> Event.ExchangeGos(a)  ) exchange in
-    let switch   = option_map (fun (a,b,c)   -> Event.SwitchGos(a,b,c)) switch in
+    let exchange = option_map (fun (a)         -> Event.ExchangeGos(a)    ) exchange in
+    let switch   = option_map (fun (a,b,c)     -> Event.SwitchGos(a,b,c)  ) switch in
     let heal     = option_map (fun (a,b,c,d,e) -> Event.HealGos(a,b,c,d,e)) heal in
-    let dbg_name = option_map (fun (a)       -> Event.DbgName(a)      ) dbg_name in
-    let fields = filter_nones [exchange;switch;heal;dbg_name] in
+    let merge    = option_map (fun (a,b,c,d)   -> Event.MergeGos(a,b,c,d) ) merge in
+    let dbg_name = option_map (fun (a)         -> Event.DbgName(a)        ) dbg_name in
+    let fields = filter_nones [exchange;switch;heal;merge;dbg_name] in
     match fields with
     | [] -> None
     | _ ->
@@ -145,18 +151,17 @@ let f alarm ranking bot_nomsg ((ls,vs) as vf) up_hdlr =
       let disable () = Transport.disable trans in
       (gossip,disable)
     ) else (
-      let gossip _ _ _ = failwith "non-coord gossipping" ; () in
+      let gossip _ _ _ _ = failwith "non-coord gossipping" ; () in
       (gossip,ident)
     )
   in
 
-  let (primary_cast,primary_send,primary_merge,primary_disable) =
+  let (primary_cast,primary_send,primary_disable) =
     let receive kind rank secure =
       if secure then (
 	let ut = match kind with
 	| Conn.Send  -> Event.ESend
 	| Conn.Cast  -> Event.ECast
-	| Conn.Other -> Event.EMergeRequest
 	in
 	let debug = addinfo (Event.string_of_type ut) name in
 	let handler msg seqno iov =
@@ -174,11 +179,10 @@ let f alarm ranking bot_nomsg ((ls,vs) as vf) up_hdlr =
 
     let trans = Transport.f alarm ranking vf Stack_id.Primary router receive in
 
-    let merge = Transport.merge trans in
     let cast = Transport.cast trans in
     let send = Transport.send trans in
     let disable () = Transport.disable trans in
-    (cast,send,merge,disable)
+    (cast,send,disable)
   in
   let sends = Array.create ls.nmembers None in
   
@@ -218,15 +222,6 @@ let f alarm ranking bot_nomsg ((ls,vs) as vf) up_hdlr =
       | _ ->
 	  send (Some (pack(msg))) (-1) iov 
     )
-  | Event.EMergeRequest
-  | Event.EMergeGranted
-  | Event.EMergeDenied ->
-      (* Note: we do not free the event because
-       * the merge operation will take the iov reference.
-       *)
-      let contact,view_id = Event.getContact ev in
-      let iov = Event.getIov ev in
-      primary_merge view_id contact (Some(pack msg)) (-1) iov ;
   | Event.EExit -> 
       primary_disable () ;
       gossip_disable () ;
@@ -239,25 +234,41 @@ let f alarm ranking bot_nomsg ((ls,vs) as vf) up_hdlr =
 	| None -> ()
 	| Some iov ->
 	    let iov = Iovecl.take name iov in
-	    gossip_gossip None (-1) iov
+	    gossip_gossip None None (-1) iov
       end ;
       Event.free name ev
 
-  | Event.EPrivateEnc -> 
-      let dst_addr,clear_text = Event.getPrivateEnc ev in
-      Auth.bckgr_ticket (Lazy.force sec_sim) ls.addr dst_addr clear_text alarm 
-  	(function cipher_text -> 
-	  up_hdlr (Event.create name Event.EPrivateEncrypted [
-	    Event.PrivateEncrypted cipher_text]
-	  ) msg)
+  | Event.EGossipExtDir ->
+      let dest = Event.getAddress ev in
+      begin
+	match pack_gossip ev with
+	| None -> ()
+	| Some iov ->
+	    let iov = Iovecl.take name iov in
+	    gossip_gossip (Some dest) None (-1) iov
+      end ;
+      Event.free name ev
 
-  | Event.EPrivateDec -> 
-      let dst_addr,cipher_text = Event.getPrivateDec ev in
-      Auth.bckgr_check (Lazy.force sec_sim) ls.addr dst_addr cipher_text alarm
-	(function clear_text -> 
-	  up_hdlr (Event.create name Event.EPrivateDecrypted [
-	    Event.PrivateDecrypted clear_text
-	  ]) msg) 
+  | Event.EAuth -> (
+      log (fun () -> "EAuth");
+      let dst_addr,op = Event.getAuthData ev in
+      match op with 
+	| Auth.Clear None -> failwith "Can't sign None clear_text"
+	| Auth.Clear (Some clear_text) -> 
+	    Auth.bckgr_ticket (Lazy.force sec_sim) ls.addr dst_addr clear_text alarm 
+  	      (function cipher_text -> 
+		up_hdlr (Event.create name Event.EAuth [
+		  Event.AuthData (dst_addr, Auth.Ticket cipher_text)]
+		) msg)
+	      
+	| Auth.Ticket None -> failwith "Can't sign a None ticket"
+	| Auth.Ticket (Some cipher_text) -> 
+	    Auth.bckgr_check (Lazy.force sec_sim) ls.addr dst_addr cipher_text alarm
+	      (function clear_text -> 
+		up_hdlr (Event.create name Event.EAuth [
+		  Event.AuthData (dst_addr, Auth.Clear clear_text)
+		]) msg) 
+    )	      
 
   | _ -> failwith "sanity:bad dn"
   in
