@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.70p1)
- *  Copyright 2000 Cornell University
+ *  Ensemble, 1.10
+ *  Copyright 2001 Cornell University, Hebrew University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -35,19 +35,3 @@ val client :
   Hsys.socket ->
   'a conn ->
   'a
-
-
-(* These are the same as above, except that the inet & port
- * of the client to the server instead of just a debugging
- * string.  
- *)
-type 'a conn_gen =
-  (Hsys.inet -> Hsys.port -> (Iovecl.t -> unit) ->
-  ((Iovecl.t -> unit) * (unit -> unit) * 'a))
-
-val server_gen : 
-  debug -> 
-  Alarm.t ->
-  port ->
-  unit conn_gen ->
-  unit
