@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.70p1)
- *  Copyright 2000 Cornell University
+ *  Ensemble, 1.10
+ *  Copyright 2001 Cornell University, Hebrew University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -43,7 +43,6 @@ val f2 :
 val disable	: 'msg t -> unit	(* disable the transport*)
 val send	: 'msg t -> rank -> 'msg (* send on the transport *)
 val cast	: 'msg t -> 'msg	(* cast on the transport *)
-val gossip	: 'msg t -> 'msg	(* gossip on the transport *)
-val merge	: 'msg t -> View.id option -> Endpt.full -> 'msg
+val gossip	: 'msg t -> Addr.set option -> 'msg (* gossip on the transport *)
 
 (**************************************************************)
