@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.70p1)
- *  Copyright 2000 Cornell University
+ *  Ensemble, 1.10
+ *  Copyright 2001 Cornell University, Hebrew University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -152,7 +152,7 @@ let hdlrs s ((ls,vs) as vf) {up_out=up;upnm_out=upnm;dn_out=dn;dnlm_out=dnlm;dnn
       then (
 	s.bcasted <- true ;
 	log (fun () -> "bcasting(recv)") ;
-	dnnm (create name EGossipExt[(gossip_send ())])
+	dnnm (create name EGossipExtDir[Address(snd bc_con) ; gossip_send ()])
       )
     ) else (
       log (fun () -> sprintf "gossip dropped from %s" (Endpt.string_of_id bc_coord)) ;
