@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.70p1)
- *  Copyright 2000 Cornell University
+ *  Ensemble, 1.10
+ *  Copyright 2001 Cornell University, Hebrew University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -40,7 +40,7 @@ let hdlrs () (ls,vs) {up_out=up;upnm_out=upnm;dn_out=dn;dnlm_out=dnlm;dnnm_out=d
   | _, NoHdr -> up ev abv
   | _, _     -> failwith bad_up_event
 
-  and uplm_hdlr ev hdr = failwith "bad uplm event"
+  and uplm_hdlr ev hdr = failwith unknown_local
   and upnm_hdlr = upnm
 
   and dn_hdlr ev abv = match getType ev with
