@@ -1,13 +1,13 @@
 /**************************************************************/
 /*
- *  Ensemble, (Version 0.70p1)
- *  Copyright 2000 Cornell University
+ *  Ensemble, 1.10
+ *  Copyright 2001 Cornell University, Hebrew University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
  */
 /**************************************************************/
-// $Header: /cvsroot/ensemble/maestro/maestro-nt/Maestro_ClSv.h,v 1.3 1999/04/01 17:04:23 tclark Exp $
+// $Header: /cvsroot/ensemble/maestro/maestro-nt/Maestro_ClSv.h,v 1.4 2001/07/31 13:23:43 orodeh Exp $
 
 #ifndef __MAESTRO_CLT_SVR_H__
 #define __MAESTRO_CLT_SVR_H__
@@ -20,7 +20,6 @@
  *                                                                  *
  ********************************************************************/
 
-#pragma interface
 #include "Maestro_GroupMember.h"
 
 /****************************************************************************/
