@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.70p1)
- *  Copyright 2000 Cornell University
+ *  Ensemble, 1.10
+ *  Copyright 2001 Cornell University, Hebrew University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -17,7 +17,7 @@ open Util
 open Shared
 (**************************************************************)
 let name = Trace.file "SIGNED"
-let failwith = Trace.make_failwith name
+let failwith s = Trace.make_failwith name s
 let log = Trace.log name
 (**************************************************************)
 
@@ -29,7 +29,9 @@ let f mbuf =
   let (marshal,_,unmarshal) = Mbuf.make_marsh name mbuf in
 
   let recv pack key secureh insecureh = 
-    let key = Security.buf_of_key key in
+    let key = match key with 
+      | Security.NoKey -> failwith "recv: NoKey"
+      | Security.Common key' -> Security.buf_of_mac key'.Security.mac in 
     let handler rbuf ofs len =
       if len <|| hdr_len then (
 	Route.drop (fun () -> sprintf "%s:size below minimum:len=%d\n" name (int_of_len len)) ;
@@ -41,7 +43,11 @@ let f mbuf =
 	  Route.drop (fun () -> sprintf "%s:rest of Conn.id did not match" name) ;
 	) else (
 	  (* Calculate the signature for the message
+	   * Use md5_init_full, so as to conform to RFC2104 (HMAC standard).
+	   * This means initialize the MD5 context to a cryptographically
+	   * strong random key. 
 	   *)
+	  (*
 	  let sign_cpt =
 	    let ctx = Hsys.md5_init () in
 	    Buf.md5_update ctx buf ofs (len -|| hdr_len) ;
@@ -49,6 +55,14 @@ let f mbuf =
 	    Buf.md5_update ctx key len0 md5len ;
 	    Buf.md5_final ctx
 	  in
+	    *)
+
+	  let sign_cpt =
+	    let ctx = Hsys.md5_init_full (Buf.break key) in
+	    Buf.md5_update ctx buf ofs (len -|| hdr_len) ;
+	    Buf.md5_update ctx buf (ofs +|| len -|| md5len_plus_8) md5len_plus_8 ;
+	    Buf.md5_final ctx
+	  in
 
 	  (* Get the integer value and length of
 	   * of the marshalled portion of the message.
@@ -109,7 +123,9 @@ let f mbuf =
   let blast (_,_,xmitvs) key pack conn =
     let ints_s = Buf.create len8 in
     let suffix = Buf.append ints_s pack in
-    let key = Security.buf_of_key key in
+    let key = match key with 
+      | Security.NoKey -> failwith "send: NoKey"
+      | Security.Common key' -> Security.buf_of_mac key'.Security.mac in 
 
     let xmit mo mi mv =
       let mo = match mo with
@@ -123,6 +139,7 @@ let f mbuf =
 
       (* Calculate the signature.
        *)
+      (*
       let sign =
 	let ctx = Hsys.md5_init () in
 	Iovecl.md5_update name ctx mv ;
@@ -132,6 +149,15 @@ let f mbuf =
 	let sign = Buf.md5_final ctx in
 	Mbuf.allocl name mbuf sign len0 md5len
       in
+      *)
+      let sign =
+	let ctx = Hsys.md5_init_full (Buf.break key) in
+	Iovecl.md5_update name ctx mv ;
+	Iovecl.md5_update name ctx mo ;
+	Buf.md5_update ctx suffix len0 (Buf.length suffix) ;
+	let sign = Buf.md5_final ctx in
+	Mbuf.allocl name mbuf sign len0 md5len
+      in
       let iovl = Iovecl.concat name [mv;mo;sign] in
       xmitvs iovl suffix
     in xmit
