@@ -1,7 +1,7 @@
 /**************************************************************/
 /*
- *  Ensemble, (Version 0.70p1)
- *  Copyright 2000 Cornell University
+ *  Ensemble, 1.10
+ *  Copyright 2001 Cornell University, Hebrew University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -59,7 +59,7 @@ static DWORD skt_input_thread(LPDWORD p){
  * thread that sends messages to this socket.
  */
 value skt_start_input(value unit){
-  int i, n, fromlen;
+  int fromlen;
   char buf[SKT_BUFSIZE];
   DWORD tid;
   struct sockaddr_in from;
