@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.70p1)
- *  Copyright 2000 Cornell University
+ *  Ensemble, 1.10
+ *  Copyright 2001 Cornell University, Hebrew University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -94,3 +94,11 @@ val prefer : (id -> int) -> id Arrayf.t -> id
 val error : id Arrayf.t -> unit
 
 val modes_of_view : set Arrayf.t -> id Arrayf.t
+
+(**************************************************************)
+(* Support for safe marshaling
+ *)
+val safe_string_of_set : set -> string
+val safe_set_of_string : string -> set 
+(**************************************************************)
+      
