@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.70p1)
- *  Copyright 2000 Cornell University
+ *  Ensemble, 1.10
+ *  Copyright 2001 Cornell University, Hebrew University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -34,6 +34,7 @@ type id =
   | Privacy				(* encryption of application data *)
   | Rekey				(* support for rekeying the group *)
   | OptRekey				(* optimized rekeying protocol *)
+  | DiamRekey                           (* Diamond rekey algorithm *)
   | Primary				(* primary partition detection *)
   | Local				(* local delivery of messages *)
   | Slander				(* members share failure suspiciions *)
