@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.70p1)
- *  Copyright 2000 Cornell University
+ *  Ensemble, 1.10
+ *  Copyright 2001 Cornell University, Hebrew University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -19,7 +19,7 @@ let atol = Arrayf.to_list
 
 (* Arrayf.sort checks first if the array is ordered.
  *)
-let sorta a = Arrayf.sort (>=) a
+let sorta a = Arrayf.sort compare a
 let sort a = atol (sorta (ltoa a))
 
 (**************************************************************)
@@ -71,7 +71,7 @@ let intersect l1 l2 =
 type 'a t = 'a Arrayf.t
 
 let collapsea a = 
-  assert (Arrayf.ordered (>=) a) ;
+  assert (Arrayf.ordered compare a) ;
   let l = Arrayf.length a in
   
   (* Handle empty case first.
@@ -113,7 +113,7 @@ let collapsea a =
 
 let inject a =
   let a = sorta a in
-  assert (Arrayf.ordered (>=) a) ;
+  assert (Arrayf.ordered compare a) ;
   collapsea a
 
 let project = Util.ident
