@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.70p1)
- *  Copyright 2000 Cornell University
+ *  Ensemble, 1.10
+ *  Copyright 2001 Cornell University, Hebrew University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -17,7 +17,7 @@ type ('key,'data) t
 (* The 'key arg is the zero value for the keys.
  * The function is a test for greater-or-equal.
  *)
-val create 	: 'key -> 'data -> ('key -> 'key -> bool) -> ('key,'data) t
+val create 	: 'key -> 'data -> ('key -> 'key -> int) -> ('key,'data) t
 
 (* Add an item.
  *)
