@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.70p1)
- *  Copyright 2000 Cornell University
+ *  Ensemble, 1.10
+ *  Copyright 2001 Cornell University, Hebrew University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -109,3 +109,8 @@ let rec clean f q =
   | None -> ()
   | Some v -> f v ; clean f q
 
+let transfer q0 q1 =
+  while not (empty q1) do
+    let v = take q1 in
+    add v q0
+  done
