@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.70p1)
- *  Copyright 2000 Cornell University
+ *  Ensemble, 1.10
+ *  Copyright 2001 Cornell University, Hebrew University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -48,3 +48,4 @@ val takeopt : 'a t -> 'a option
 val to_list : 'a t -> 'a list
 val to_array : 'a t -> 'a array
 val clean : ('a -> unit) -> 'a t -> unit
+val transfer : 'a t -> 'a t -> unit
