@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.70p1)
- *  Copyright 2000 Cornell University
+ *  Ensemble, 1.10
+ *  Copyright 2001 Cornell University, Hebrew University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -36,7 +36,7 @@ module type OrderedType =
     type t
     val zero : t
     val ge : t -> t -> bool
+    val cmp : t -> t -> int
   end
 
-
 (**************************************************************)
