@@ -1,7 +1,7 @@
 (**************************************************************)
 (*
- *  Ensemble, (Version 0.70p1)
- *  Copyright 2000 Cornell University
+ *  Ensemble, 1.10
+ *  Copyright 2001 Cornell University, Hebrew University
  *  All rights reserved.
  *
  *  See ensemble/doc/license.txt for further information.
@@ -35,7 +35,9 @@ module type OrderedType =
   sig
     type t
     val zero : t
+    (*val ge : t -> t -> bool*)
     val ge : t -> t -> bool
+    val cmp : t -> t -> int
   end
 
 (**************************************************************)
